<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use DateTime;

class Pla01900ParamCozCentral extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $crudService;
    protected $entityManager;
    protected $util;
    protected $Coz01000ConsumoPlanejado;
    protected $novoCodigo;
    protected $environment;
    
    public function __construct($crudService, $parameterBag, $entityManager, $util, $Coz01000ConsumoPlanejado, $novoCodigo, $environment){
        parent::__construct($crudService, $parameterBag);
        $this->entityManager            = $entityManager;
        $this->util                     = $util;
        $this->Coz01000ConsumoPlanejado = $Coz01000ConsumoPlanejado;
        $this->novoCodigo               = $novoCodigo;
        $this->environment              = $environment;
    }
    
    public function validaCadastro(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrorg = $row['NRORG'];

        if ($row['CENTRALCELULAPROD'] == 'S') { //validações central
            $retornoCentral = $this->validaCentral($request, $response);
            if(count($retornoCentral) > 0) {
                foreach($retornoCentral as $pos) {
                    if ($pos['CDSERVRETPME'] == NULL && $pos['CDTPRETIPADR'] == NULL) $msg[] = 'A Central ' . $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL'] . ' não possui serviço para retirada automática e tipo de retirada default cadastrados.';
                    else {
                        if ($pos['CDSERVRETPME'] == NULL) $msg[] = 'A Central ' . $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL'] . ' não possui serviço para retirada automática cadastrado.';
                        if ($pos['CDSERVRETPME'] == NULL) $msg[] = 'A Central ' . $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL'] . ' não possui tipo de retirada default cadastrado.';
                    }
                    
                    if ($pos['CDSERVRETPME'] != NULL && $pos['IDSERVATIV'] == 'N')  $msg[] = 'O serviço de retirada automática da central ' . $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL'] . ' não está habilitado para realizar previsão, requisição e retirada.';
                    if ($pos['CDTPDISTRSAID'] == NULL) $msg[] = 'A central ' . $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL'] . ' não possui Tipo de operação de saída cadastrado para a realização da transferência de estoque.';
                }

                $nrLista = $this->geraListaUnidade($row, $nrorg); //gera lista das unidades 
                
                $retornoListaUnid = $this->validaUnidAssociadaCentral($nrLista);
                if (count($retornoListaUnid) > 0) {
                    foreach($retornoListaUnid as $pos) {
                        $msg[] = 'A unidade ' . $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL'] . ' não possui central e nem célula produtora.';
                    }
                }
                    
            } else $msg[] = 'Não há unidade cadastrada como "central" do tipo "Cozinha Central".';
        }
        
        //Monta lista de servico
        if ($row['CDSERVFILI'] == 'T'){
            $servicos = $this->todosServicos($row, $nrorg);
            $nrListaServ =  $this->Coz01000ConsumoPlanejado->gerarListaServ($servicos, $nrorg);
        } else $nrListaServ =  $this->Coz01000ConsumoPlanejado->gerarListaServ($row['CDSERVFILI'], $nrorg);
        
        if ($row['SERVICOVALID'] == 'S') { //validacoes servico
            $retornoListaServ = $this->validaServico($nrListaServ);

            foreach($retornoListaServ as $pos) {
                if ($pos['IDSERVTRAN'] == 'N') $msg[] = 'O serviço ' . $pos['CDSERVFILI'] . ' - ' . $pos['NMSERVICO'] . ' da unidade ' . $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL'] . ' não está definido como transportado.';
                if ($pos['IDSERVATIV'] == 'N') $msg[] = 'O serviço transportado ' . $pos['CDSERVFILI'] . ' - ' . $pos['NMSERVICO'] . ' da unidade ' .  $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL'] . ' não está habilitado para realizar previsão, requisição e retirada.';
            }
        }
        
        if ($row['PRODACABADO'] == 'S') { //validacoes produto acabado
            $resultProdAcabPerCap = $this->validaProdAcabPerCap($nrListaServ, $row);
            foreach ($resultProdAcabPerCap as $pos) {
                $tipoRece = '';
                if ($pos['IDTIPORECE'] == 'N') $tipoRece = 'Nenhum';
                if ($pos['IDTIPORECE'] == 'P') $tipoRece = 'Padrão';
                if ($pos['IDTIPORECE'] == 'F' || $pos['IDTIPORECE'] == 'L') $tipoRece = 'Local ' . '(Unidade: ' . $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL'] . ')'; 
                if ($pos['IDTIPORECE'] == 'T') $tipoRece = 'Tipo de Cardápio';
                if ($pos['IDTIPORECE'] == 'S') $tipoRece = 'Serviço ' . '(Unidade: ' . $pos['CDFILIAL'] . ' - '. $pos['NMFILIAL'] . ' | Serviço: ' . $pos['CDSERVFILI'] . ' - '.$pos['NMSERVICO'] .')'; 
                if ($pos['CDPRATO'] && $pos['CDPRODACABADO'] == NULL) $msg[] = 'Prato ' . $pos['CDPRATO'] . ' - ' . $pos['NMPRATO'] . ' não possui produto acabado relacionado a receita '. $tipoRece . '.' ;
            }
        }
        
        if ($row["QTDPERCAP"] == 'S') { //validacoes qtd per capita
            $resultProdAcabPerCap = $this->validaProdAcabPerCap($nrListaServ, $row);
            foreach ($resultProdAcabPerCap as $pos) {
                if ($pos['QTPORPESSOA'] == NULL || $pos['QTPORPESSOA'] == '0') $msg[] = 'O prato ' . $pos['CDPRATO'] . ' - ' . $pos['NMPRATO'] . ' não possui a quantidade por pessoa calculado.' ;
            }
        }
        
        if (isset($msg) == true) $response->addDataset(new DataSet('msg', array_unique($msg)));
        else  {
            $sucesso[] = 'Validação realizada com sucesso.';
            $response->addDataset(new DataSet('sucesso', array_unique($sucesso)));
        }
    }
    
    public function validaCentral($request, $response) {
	    $sql = "SELECT F.CDFILIAL, F.NMFILIAL, P.CDTPRETIPADR, P.CDSERVRETPME, S.NMSERVICO, S.IDSERVATIV, P.CDTPDISTRSAID FROM FILIAL F INNER JOIN PARAMPLANEJA P ON P.CDFILIAL = F.CDFILIAL LEFT JOIN SERVICO S ON S.CDFILIAL = P.CDFILIAL AND S.CDSERVFILI = P.CDSERVRETPME WHERE F.IDTPFILIALCENT = 'C'";
        
        $connection = $this->entityManager->getConnection();
        $result = $connection->fetchAll($sql);
        return $result;
	}
	
	public function validaUnidAssociadaCentral($nrLista) {
	    $sql = "SELECT F.CDFILIAL, F.NMFILIAL 
                  FROM FILIAL F 
                 INNER JOIN LISTA L ON L.NRLISTA = :P_NRLISTA AND L.CDLISTA = F.CDFILIAL 
                  LEFT JOIN FILIAL C ON C.CDFILIAL = F.CDFILCENTPME AND C.IDTPFILIALCENT = 'C' 
                 WHERE F.CDFILCENTPME IS NULL
                   AND F.CDFILIAL NOT IN (SELECT DISTINCT CDFILIAGREGA 
                                            FROM LISTA, CELULAPROD 
                                           WHERE NRLISTA = :P_NRLISTA 
                                             AND CDFILIAGREGA = CDLISTA)
                 UNION 
                SELECT L.CDLISTA, F.NMFILIAL 
                  FROM LISTA L 
                 INNER JOIN FILIAL F ON F.CDFILIAL = L.CDLISTA 
                  LEFT JOIN CELULAPROD C ON C.CDFILIAGREGA = L.CDLISTA 
                 WHERE L.NRLISTA = :P_NRLISTA AND C.CDFILIAGREGA IS NULL 
                   AND L.CDLISTA NOT IN (SELECT CDFILIAL FROM FILIAL WHERE CDFILCENTPME IS NOT NULL)";
	    $params = array (
            "P_NRLISTA" => $nrLista
        );
        $validUnid = $this->entityManager->getConnection()->fetchAll($sql, $params);   
        return $validUnid;
	}
	
	public function validaServico($nrListaServ) {
	    $sql = "SELECT L.CDFILIAL, F.NMFILIAL, L.CDSERVFILI, L.NMSERVICO, S.IDSERVATIV, S.IDSERVTRAN FROM LISTA_SERVICO L INNER JOIN SERVICO S ON S.CDFILIAL = L.CDFILIAL AND S.CDSERVFILI = L.CDSERVFILI INNER JOIN FILIAL F ON F.CDFILIAL = L.CDFILIAL WHERE L.NRLISTA = :P_NRLISTASERV";
	    $params = array (
            "P_NRLISTASERV" => $nrListaServ
        );
        $validServ = $this->entityManager->getConnection()->fetchAll($sql, $params);   
        return $validServ;
	}
	
	public function validaProdAcabPerCap($nrListaServ, $row) {
	    $sql = "SELECT C.CDFILIAL, C.CDSERVFILI, C.CDTIPOPRAT, R.CDPRATO, C.NMPRATO, R.CDFILAUXPRAT, R.IDTIPORECE, R.CDPRODPRODUCAO                    AS CDPRODACABADO, MAX(ROUND(R.QTPADRAO, 5))           AS QTPORPESSOA, C.NMFILIAL, C.NMSERVICO FROM ( SELECT C.CDFILIAL, C.CDSERVFILI, C.DTCARDAPIO, P.CDTIPOPRAT, P.CDFILAUXPRAT, P.CDPRATO, PT.NMPRATO, F.NMFILIAL, S.NMSERVICO FROM CARDADIA C INNER JOIN LISTA_SERVICO  L ON L.NRLISTA = :P_NRLISTASERV AND C.CDFILIAL = L.CDFILIAL AND C.CDSERVFILI = L.CDSERVFILI INNER JOIN COMPCARD       P ON C.CDFILIAL = P.CDFILIAL AND P.NRCARDAPIO = C.NRCARDAPIO AND C.DTCARDAPIO BETWEEN :P_DTINICARD AND :P_DTFINCARD INNER JOIN PRATO          PT ON PT.CDPRATO = P.CDPRATO AND PT.CDFILAUXPRAT = P.CDFILAUXPRAT INNER JOIN FILIAL F ON F.CDFILIAL = C.CDFILIAL INNER JOIN SERVICO S ON S.CDFILIAL = C.CDFILIAL AND S.CDSERVFILI = C.CDSERVFILI WHERE NVL(C.QTCLIESTICAR, 0) > 0 AND C.DTCARDAPIO BETWEEN :P_DTINICARD AND :P_DTFINCARD )                             C LEFT JOIN TABLE ( RECEITA_UTIL(C.CDFILIAL, C.CDSERVFILI, C.CDPRATO, C.CDFILAUXPRAT, '0', 'Z', NULL, 0, 0, 'S') )           R ON R.CDPRATO = C.CDPRATO AND R.CDFILAUXPRAT = C.CDFILAUXPRAT GROUP BY C.CDFILIAL, C.CDSERVFILI, C.CDTIPOPRAT, C.NMPRATO, R.CDPRATO, R.CDFILAUXPRAT, R.IDTIPORECE, R.CDPRODPRODUCAO, C.NMFILIAL, C.NMSERVICO ORDER BY C.CDFILIAL, C.CDSERVFILI, C.CDTIPOPRAT, R.CDPRATO, R.CDFILAUXPRAT";
	    $params = array (
            "P_NRLISTASERV" => $nrListaServ,
            "P_DTINICARD"   => $row['DATAINICIAL'],
            "P_DTFINCARD"   => $row['DATAFINAL']
        );
        $prodAcabPerCap = $this->entityManager->getConnection()->fetchAll($sql, $params);  
        return $prodAcabPerCap;
	}
	
	public function todasUnidades($row, $nrorg){
	    $sql = "SELECT DISTINCT F.CDFILIAL, F.NMFILIAL, F.NRORG FROM FILIAL       F LEFT JOIN FILGRUPFILI  FG ON F.CDFILIAL = FG.CDFILIAL WHERE ( ( FG.CDGRUPFILI = :CDGRUPFILI ) OR ( :CDGRUPFILI IS NULL ) OR ( :CDGRUPFILI = 'T' ) ) AND F.IDCNTRRESC = 'N' AND F.NRORG = :NRORG ORDER BY F.CDFILIAL";
	    $params = array (
            "NRORG"      => $nrorg,
            "CDGRUPFILI" => $row['CDGRUPFILI']
        );
	    $result = $this->entityManager->getConnection()->fetchAll($sql, $params);  
        return $result;
	}
	
	public function todosServicos($row, $nrorg){ 
	    $sql = "SELECT DISTINCT ( S.CDFILIAL || S.CDSERVFILI ) AS CDFILISERV, S.CDFILIAL, F.NMFILIAL, S.CDSERVFILI, S.NMSERVICO, S.NRORG FROM SERVICO S JOIN FILIAL F ON S.CDFILIAL = F.CDFILIAL LEFT JOIN FILGRUPFILI FG ON S.CDFILIAL = FG.CDFILIAL AND (FG.CDGRUPFILI = :CDGRUPFILI OR :CDGRUPFILI = 'T') WHERE ( ( INSTR(:CDFILIAL, S.CDFILIAL) > 0 ) OR (:CDFILIAL = 'T' ) ) AND ( ( INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0 ) OR (:CDGRUPSERV = 'T' ) ) AND ( IDTIPOSERV = 'N' ) ORDER BY CDFILIAL, CDSERVFILI";
	    if (is_array($row['CDFILIAL'])) $unidade = implode("_", $row['CDFILIAL']);
	    else $unidade = $row['CDFILIAL'];
	    $params = array (
	        "CDGRUPSERV" => $row['CDGRUPSERV'],
            "CDFILIAL"   => $unidade,
            "CDGRUPFILI" => $row['CDGRUPFILI']
        );
	    $result = $this->entityManager->getConnection()->fetchAll($sql, $params);  
        return $result;
	}
	
	public function geraListaUnidade($row, $nrorg) {
	    $nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $this->environment->getNrorgTrab());
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        if(empty($row['NMFILIAL']) == true || $row['NMFILIAL'] == 'T') {
            $unidade = $this->todasUnidades($row, $nrorg);
            foreach ($unidade as $pos) {
    			$stmt = $this->entityManager->getConnection()->prepare($sql);
    			$stmt->execute(
                        array(
                            "NRLISTA" => $nrlista,
                            "CDLISTA" => $pos['CDFILIAL']
                        )
                    );
    		}
        } else {
    		foreach ($row['NMFILIAL'] as $pos) {
    			$stmt = $this->entityManager->getConnection()->prepare($sql);
    			$stmt->execute(
                        array(
                            "NRLISTA" => $nrlista,
                            "CDLISTA" => $pos
                        )
                    );
    		}
        }
		return $nrlista;
	}
}